@@ -15,7 +15,7 @@ from restclients_core.exceptions import DataFailureException
 from uw_gws.dao import GWS_DAO
 from uw_gws.models import (
     Group, CourseGroup, GroupReference, GroupEntity, GroupMember,
-    GroupAffiliate)
+    GroupAffiliate, GroupHistory)
 from uw_gws.exceptions import InvalidGroupID
 
 
@@ -198,6 +198,34 @@ class GWS(object):
             return errors[0].get("notFound", [])
         return []
 
+    def get_group_history(self, group_id,
+                          activity=None,
+                          start=0,
+                          id=None):
+        """
+        Returns a list of GroupHistory objects, in the order of
+        from the earliest to the latest.
+        :param start: Epoch timestamp in seconds
+        :param activity: one of {"acl", "membership"}
+        :param id: member ID selector
+        """
+        kwargs = {}
+        if activity:
+            kwargs['activity'] = activity
+        if start:
+            kwargs['start'] = start * 1000
+        if id:
+            kwargs['id'] = id
+        url = "{}/group/{}/history".format(self.API, group_id)
+        self._valid_group_id(group_id)
+        if len(kwargs):
+            url = "{}?{}".format(url, urlencode(kwargs))
+        data = self._get_resource(url)
+        changes = []
+        for datum in data.get("data"):
+            changes.insert(0, GroupHistory(data=datum))
+        return changes
+
     def get_effective_members(self, group_id):
         """
         Returns a list of effective restclients.GroupMember objects for the
@@ -233,14 +261,22 @@ class GWS(object):
         """
         Returns True if the netid is in the group, False otherwise.
         """
+        return self.is_member(group_id, netid, True)
+
+    def is_direct_member(self, group_id, netid):
+        """
+        Returns True if the netid is in the group, False otherwise.
+        """
+        return self.is_member(group_id, netid, False)
+
+    def is_member(self, group_id, netid, is_effective):
         self._valid_group_id(group_id)
 
         # GWS doesn't accept EPPNs on effective member checks, for UW users
         netid = re.sub('@washington.edu', '', netid)
-
-        url = "{}/group/{}/effective_member/{}".format(self.API,
-                                                       group_id,
-                                                       netid)
+        url = "{}/group/{}/{}/{}".format(
+            self.API, group_id,
+            "effective_member" if is_effective else "member", netid)
 
         # Not using _get_resource() here because it automatically logs 404s
         response = self.DAO.getURL(url, self._headers())
